#include <GL/glut.h>
#include "defs.h"



// GLOBALS 
Scene* scene = NULL;
RayCamera* camera = NULL;
unsigned char* imageData = NULL;
int width = 200;
int height = 200;
int startTime;



//------------------------------------------------------------------------------
void display()
{
  glViewport(0, 0, width, height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, width, 0, height);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glRasterPos2i(0,0);
  glDrawPixels(width, height, GL_RGB, GL_UNSIGNED_BYTE, imageData);
  glutSwapBuffers();
}

//------------------------------------------------------------------------------
void renderLine(int y)
{
  if (y == 0)
  {
    printf( "Starting Rendering Thread...\n" );
    startTime = glutGet( GLUT_ELAPSED_TIME );
  }
  float aspect = float(width)/height;
  float v = float(y)/height;
  for( int x=0; x < width; x++ )
  {
    float u = aspect*x/width;
    RGBColor pix = camera->renderPixel(u,v,scene);
    if( pix[0] > 1 ) pix[0] = 1;
    if( pix[1] > 1 ) pix[1] = 1;
    if( pix[2] > 1 ) pix[2] = 1;    
    unsigned char* pos = imageData + ((height-1 - y) * width + x) * 3;
    *pos++ = unsigned char(255 * pix[0]);
    *pos++ = unsigned char(255 * pix[1]);
    *pos++ = unsigned char(255 * pix[2]);    
  }
  y += 1;
  if (y == height)
    printf( "Rendering time = %6.1f secs\n", (glutGet(GLUT_ELAPSED_TIME) - startTime)/1000.0 );
  else
    glutTimerFunc(1, renderLine, y);
  glutPostRedisplay();
}

//------------------------------------------------------------------------------
void mouse( int button, int state, int x, int y )
{
  // trace the clicked on pixel
  if( state == GLUT_DOWN )
  {
    float aspect = float(width)/height;
    float v = float(y)/height;
    float u = aspect*x/width;
    RGBColor pix = camera->renderPixel(u,v,scene);

    printf("Color %f %f %f\n", pix[0], pix[1], pix[2] );
  }  
}

//------------------------------------------------------------------------------
void main( int argc, char* argv[] )
{
  if( argc < 2 )
  {
    printf("Specify scene definition file\n" );
    return;
  }
  glutInit( &argc, argv );
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize(width, height);
  glutInitWindowPosition(100, 100);
  glutCreateWindow("Toy RayTracer");
  glutDisplayFunc(display);
  glutMouseFunc( mouse );

  imageData = new unsigned char[width*height*3];
  Parse(argv[1]);
  glutTimerFunc(0, renderLine, 0);

  glutMainLoop();
}

