#include "sphere.h"

//------------------------------------------------------------------------------
bool Sphere::intersect( Ray& ray )
{
  vec3f c = center - ray.origin;
  float v = ray.dir.dot(c);

  // Check if there is even a chance an
  // intersection might be closer
  if (v - radius > ray.t)
    return false;

  // Test if the ray intersects sphere
  float t = radiusSqr + v*v - c.dot(c);
  if (t < 0)
    return false;

  // Test if intersection is in the positive
  // ray direction and is the closest so far
  t = v - sqrt(t);
  if ((t > ray.t) || (t < 0))
    return false;

  // store the intesection in the ray
  ray.t = t;
  ray.obj = this;
  return true;
}


//------------------------------------------------------------------------------
RGBColor Sphere::shade( const Ray& ray, Scene* scene )
{
  // An object shader doesn't really do too much other than
  // supply a few critical bits of geometric information
  // for a surface shader. It must must compute:
  //
  //   1. the point of intersection (p)
  //   2. a unit-length surface normal (n)
  //   3. a unit-length vector towards the ray's origin (v)
  //
  vec3f P = ray.origin + ray.dir * ray.t;
  vec3f N = normalize(P - center);

  // The illumination model is applied
  // by the surface's shade() method
  return surface->shade( P, N, ray, scene );
}
