#include "defs.h"

//------------------------------------------------------------------------------
RGBColor Surface::shade( const vec3f& P, const vec3f& N, const Ray& ray, 
    Scene* scene )
{
  RGBColor color(0,0,0);

  if( ray.depth >= 5 )
    return color; // terminate recursion

  // accumulate contribution from the lights
  vec3f V = -ray.dir;
  for( int l=0; l < (int)scene->lights.size(); l++ )
  {
    Light* light = scene->lights[l];
    if( light->type == Light::AMBIENT )
      color += ka * baseColor * light->color;
    else
    {
      vec3f L;
      if( light->type == Light::POINT)
        L = normalize( light->v - P );
      else 
        L = -light->v;

      // Check if the surface point is in shadow
      vec3f PP = P + L * TINY;
      Ray shadowRay = Ray(PP, L, ray.depth+1 );
      if (shadowRay.trace(scene))
        continue;

      // diffuse
      float NdotL = N.dot(L);
      vec3f R = N * 2 * NdotL - L;
      if( NdotL > 0 )
      {
        if( kd > 0 )
          color += kd * baseColor * NdotL * light->color;
        if( ks > 0 )  
        {
          float VdotR = V.dot( R );
          if( VdotR > 0 )
            color += ks * pow( VdotR, ns ) * light->color;
        }
      }
    }
  }


  // Compute illumination due to reflection
  if ( kr > 0 )
  {
    float NdotV = N.dot(V);
    if( NdotV > 0 )
    {
      vec3f R =  N * 2 * NdotV - V;
      vec3f PP = P + R * TINY;
      Ray reflectedRay( PP, R, ray.depth + 1 );
      if( reflectedRay.trace( scene ) )        
        color += kr * reflectedRay.shade( scene );
      else 
        color += kr * scene->backgroundColor;
    }
    
  }

  // Compute illumination due to refraction
  if( kt > 0 )
  {
    // refaction code here
  }

  return color;
}
